@@ -1,29 +1,3 @@
-/*
-  Racket
-  Copyright (c) 2004-2018 PLT Design Inc.
-  Copyright (c) 1995-2001 Matthew Flatt, Scott Owens
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301 USA.
-
-  libscheme
-  Copyright (c) 1994 Brent Benson
-  All rights reserved.
-*/
-
-
 /* DANGER! DANGER! DANGER! DANGER! DANGER! DANGER! DANGER! DANGER!
 
    This code is fragile, due to the Small_Bignum optimization, and
@@ -406,7 +380,8 @@ int scheme_bignum_get_long_long_val(const Scheme_Object *o, mzlonglong *v)
   } else if (SCHEME_BIGLEN(o) == 0) {
     *v = 0;
     return 1;
-  } else if (SCHEME_BIGDIG(o)[MAX_BN_SIZE_FOR_LL - 1] == FIRST_BIT_MASK_LL 
+  } else if ((SCHEME_BIGLEN(o) == MAX_BN_SIZE_FOR_LL)
+             && SCHEME_BIGDIG(o)[MAX_BN_SIZE_FOR_LL - 1] == FIRST_BIT_MASK_LL 
 # if !defined(USE_LONG_LONG_FOR_BIGDIG) && !defined(SIXTY_FOUR_BIT_INTEGERS)
 	     && !SCHEME_BIGDIG(o)[0]
 # endif
@@ -417,7 +392,8 @@ int scheme_bignum_get_long_long_val(const Scheme_Object *o, mzlonglong *v)
     v2 = ((umzlonglong)v2 << 63);
     *v = v2;
     return 1;
-  } else if ((SCHEME_BIGDIG(o)[MAX_BN_SIZE_FOR_LL - 1] & FIRST_BIT_MASK_LL) != 0) { /* Won't fit into a signed long long */
+  } else if ((SCHEME_BIGLEN(o) == MAX_BN_SIZE_FOR_LL)
+             && ((SCHEME_BIGDIG(o)[MAX_BN_SIZE_FOR_LL - 1] & FIRST_BIT_MASK_LL) != 0)) { /* Won't fit into a signed long long */
     return 0;
   } else {
     mzlonglong v2;
@@ -485,7 +461,7 @@ static Scheme_Object *make_single_bigdig_result(int pos, bigdig d)
   /* May not need to allocate: */
   sm = &quick;
   sm->o.iso.so.type = scheme_bignum_type;
-  SCHEME_SET_BIGPOS(sm, pos);
+  SCHEME_INIT_BIGPOS(sm, pos);
   SCHEME_BIGLEN(sm) = 1;
   SCHEME_BIGDIG(sm) = sm->v;
   sm->v[0] = d;
@@ -950,7 +926,7 @@ static Scheme_Object *do_bitop(const Scheme_Object *a, const Scheme_Object *b, i
   intptr_t a_size, b_size, a_pos, b_pos, res_alloc, i;
   short res_pos;
   bigdig* a_digs, *b_digs, *res_digs, quick_digs[1];
-  int carry_out_a, carry_out_b, carry_out_res, carry_in_a, carry_in_b, carry_in_res;
+  int carry_out_a, carry_out_b, carry_out_res, carry_in_b, carry_in_res;
   Scheme_Object* o;
   SAFE_SPACE(asd) SAFE_SPACE(bsd)
 
@@ -996,7 +972,7 @@ static Scheme_Object *do_bitop(const Scheme_Object *a, const Scheme_Object *b, i
     res_digs = allocate_bigdig_array(res_alloc);
 
   carry_out_a = carry_out_b = carry_out_res = 1;
-  carry_in_a = carry_in_b = carry_in_res = 0;
+  carry_in_b = carry_in_res = 0;
 
   for (i = 0; i < res_alloc; ++i)
   {
@@ -1005,8 +981,8 @@ static Scheme_Object *do_bitop(const Scheme_Object *a, const Scheme_Object *b, i
     a_val = a_digs[i];
     if (!a_pos)
     {
-      /* We have to do te operation on the 2's complement of a */
-      carry_in_a = carry_out_a;
+      /* We have to do the operation on the 2's complement of a */
+      int carry_in_a = carry_out_a;
       carry_out_a = (carry_in_a == 1 && a_val == 0) ? 1 : 0;
       a_val = ~a_val + carry_in_a;
     }
