@@ -8,12 +8,14 @@ SHARED_OK Scheme_Hash_Tree *empty_hash_tree;
 
 SHARED_OK static int validate_compile_result = 0;
 SHARED_OK static int recompile_every_compile = 0;
+SHARED_OK static int show_linklets = 0;
 
 static Scheme_Object *serializable_symbol;
 static Scheme_Object *unsafe_symbol;
 static Scheme_Object *static_symbol;
 static Scheme_Object *use_prompt_symbol;
 static Scheme_Object *uninterned_literal_symbol;
+static Scheme_Object *quick_symbol;
 static Scheme_Object *constant_symbol;
 static Scheme_Object *consistent_symbol;
 static Scheme_Object *noncm_symbol;
@@ -30,6 +32,7 @@ static Scheme_Object *primitive_table(int argc, Scheme_Object **argv);
 static Scheme_Object *primitive_to_position(int argc, Scheme_Object **argv);
 static Scheme_Object *position_to_primitive(int argc, Scheme_Object **argv);
 static Scheme_Object *primitive_in_category_p(int argc, Scheme_Object **argv);
+static Scheme_Object *primitive_lookup(int argc, Scheme_Object **argv);
 
 static Scheme_Object *linklet_p(int argc, Scheme_Object **argv);
 static Scheme_Object *compile_linklet(int argc, Scheme_Object **argv);
@@ -62,7 +65,7 @@ static Scheme_Linklet *compile_and_or_optimize_linklet(Scheme_Object *form, Sche
                                                        Scheme_Object *name,
                                                        Scheme_Object **_import_keys,
                                                        Scheme_Object *get_import,
-                                                       int unsafe_mode, int static_mode);
+                                                       int unsafe_mode, int static_mode, int serializable);
 
 static Scheme_Object *_instantiate_linklet_multi(Scheme_Linklet *linklet, Scheme_Instance *instance,
                                                  int num_instances, Scheme_Instance **instances,
@@ -101,11 +104,13 @@ void scheme_init_linklet(Scheme_Startup_Env *env)
   REGISTER_SO(static_symbol);
   REGISTER_SO(use_prompt_symbol);
   REGISTER_SO(uninterned_literal_symbol);
+  REGISTER_SO(quick_symbol);
   serializable_symbol = scheme_intern_symbol("serializable");
   unsafe_symbol = scheme_intern_symbol("unsafe");
   static_symbol = scheme_intern_symbol("static");
   use_prompt_symbol = scheme_intern_symbol("use-prompt");
   uninterned_literal_symbol = scheme_intern_symbol("uninterned-literal");
+  quick_symbol = scheme_intern_symbol("quick");
 
   REGISTER_SO(constant_symbol);
   REGISTER_SO(consistent_symbol);
@@ -126,6 +131,7 @@ void scheme_init_linklet(Scheme_Startup_Env *env)
   ADD_IMMED_PRIM("primitive->compiled-position", primitive_to_position, 1, 1, env);
   ADD_IMMED_PRIM("compiled-position->primitive", position_to_primitive, 1, 1, env);
   ADD_IMMED_PRIM("primitive-in-category?", primitive_in_category_p, 2, 2, env);
+  ADD_IMMED_PRIM("primitive-lookup", primitive_lookup, 1, 1, env);
 
   ADD_FOLDING_PRIM("linklet?", linklet_p, 1, 1, 1, env);
   ADD_PRIM_W_ARITY2("compile-linklet", compile_linklet, 1, 5, 2, 2, env);
@@ -189,6 +195,9 @@ void scheme_init_linklet(Scheme_Startup_Env *env)
         recompile_every_compile = 32;
     }
   }
+
+  if (scheme_getenv("PLT_LINKLET_SHOW"))
+    show_linklets = 1;
 }
 
 void scheme_init_unsafe_linklet(Scheme_Startup_Env *env)
@@ -291,9 +300,10 @@ static Scheme_Object *primitive_in_category_p(int argc, Scheme_Object **argv)
     else if (SAME_OBJ(cat, folding_symbol))
       r = (opt >= SCHEME_PRIM_OPT_FOLDING);
     else if (SAME_OBJ(cat, omitable_symbol))
-      r = (SCHEME_PRIM_PROC_OPT_FLAGS(v) & (SCHEME_PRIM_IS_OMITABLE_ANY
-                                            | SCHEME_PRIM_IS_UNSAFE_FUNCTIONAL
-                                            | SCHEME_PRIM_IS_UNSAFE_OMITABLE));
+      r = (SCHEME_PRIM_PROC_OPT_FLAGS(v) & (SCHEME_PRIM_IS_OMITABLE
+                                            | SCHEME_PRIM_IS_OMITABLE_ALLOCATION
+                                            | SCHEME_PRIM_IS_UNSAFE_OMITABLE
+                                            | SCHEME_PRIM_IS_UNSAFE_FUNCTIONAL));
     else
       r = 0;
   } else
@@ -302,6 +312,18 @@ static Scheme_Object *primitive_in_category_p(int argc, Scheme_Object **argv)
   return (r ? scheme_true : scheme_false);
 }
 
+static Scheme_Object *primitive_lookup(int argc, Scheme_Object **argv)
+{
+  Scheme_Object *v;
+
+  if (!SCHEME_SYMBOLP(argv[0]))
+    scheme_wrong_contract("primitive-lookup", "symbol?", 0, argc, argv);
+
+  v = scheme_hash_get(scheme_startup_env->all_primitives_table, argv[0]);
+
+  return (v ? v : scheme_false);
+}
+
 static Scheme_Object *linklet_p(int argc, Scheme_Object **argv)
 {
   return (SAME_TYPE(SCHEME_TYPE(argv[0]), scheme_linklet_type)
@@ -337,7 +359,7 @@ void extract_import_info(const char *who, int argc, Scheme_Object **argv,
       if (!*_import_keys) {
         scheme_contract_error(who,
                               "no vector supplied for import keys, but import-getting function provided;\n"
-                              " the function argument must be `#f' when the vector argument is `#f'",
+                              " the function argument must be `#f` when the vector argument is `#f`",
                               "import-getting function", 1, argv[3],
                               NULL);
       }
@@ -350,7 +372,8 @@ void extract_import_info(const char *who, int argc, Scheme_Object **argv,
 
 static void parse_compile_options(const char *who, int arg_pos,
                                   int argc, Scheme_Object **argv,
-                                  int *_unsafe, int *_static_mode)
+                                  int *_unsafe, int *_static_mode,
+                                  int *_serializable)
 {
   Scheme_Object *redundant = NULL, *flag, *flags = argv[arg_pos];
   int serializable = 0;
@@ -358,6 +381,7 @@ static void parse_compile_options(const char *who, int arg_pos,
   int static_mode = *_static_mode;
   int use_prompt_mode = 0;
   int uninterned_literal_mode = 0;
+  int quick_mode = 0;
   
   while (SCHEME_PAIRP(flags)) {
     flag = SCHEME_CAR(flags);
@@ -381,6 +405,10 @@ static void parse_compile_options(const char *who, int arg_pos,
       if (uninterned_literal_mode && !redundant)
         redundant = flag;
       uninterned_literal_mode = 1;
+    } else if (SAME_OBJ(flag, quick_symbol)) {
+      if (quick_mode && !redundant)
+        redundant = flag;
+      quick_mode = 1;
     } else
       break;
     flags = SCHEME_CDR(flags);
@@ -388,7 +416,7 @@ static void parse_compile_options(const char *who, int arg_pos,
 
   if (!SCHEME_NULLP(flags))
     scheme_wrong_contract("compile-linklet",
-                          "(listof/c 'serializable 'unsafe 'static 'use-prompt 'uninterned-literal)",
+                          "(listof/c 'serializable 'unsafe 'static 'use-prompt 'uninterned-literal 'quick)",
                           arg_pos, argc, argv);
 
   if (redundant)
@@ -399,14 +427,13 @@ static void parse_compile_options(const char *who, int arg_pos,
 
   *_unsafe = unsafe;
   *_static_mode = static_mode;
+  *_serializable = serializable;
 }
 
 static Scheme_Object *compile_linklet(int argc, Scheme_Object **argv)
 {
   Scheme_Object *name, *e, *import_keys, *get_import, *a[2];
-  int unsafe = 0, static_mode = 0;
-
-  /* Last argument, `serializable?`, is ignored */
+  int unsafe = 0, static_mode = 0, serializable = 1;
 
   extract_import_info("compile-linklet", argc, argv, &import_keys, &get_import);
 
@@ -419,11 +446,18 @@ static Scheme_Object *compile_linklet(int argc, Scheme_Object **argv)
   if (!SCHEME_STXP(e))
     e = scheme_datum_to_syntax(e, scheme_false, DTS_CAN_GRAPH);
 
+  if (show_linklets) {
+    char *s;
+    intptr_t s_len;
+    s = scheme_write_to_string(scheme_syntax_to_datum(e), &s_len);
+    printf("%s\n", s);
+  }
+
   if (argc > 4)
-    parse_compile_options("compile-linklet", 4, argc, argv, &unsafe, &static_mode);
+    parse_compile_options("compile-linklet", 4, argc, argv, &unsafe, &static_mode, &serializable);
 
   e = (Scheme_Object *)compile_and_or_optimize_linklet(e, NULL, name, &import_keys, get_import,
-                                                       unsafe, static_mode);
+                                                       unsafe, static_mode, serializable);
 
   if (import_keys) {
     a[0] = e;
@@ -437,7 +471,7 @@ static Scheme_Object *recompile_linklet(int argc, Scheme_Object **argv)
 {
   Scheme_Object *name, *import_keys, *get_import, *a[2];
   Scheme_Linklet *linklet;
-  int unsafe = 0, static_mode = 0;
+  int unsafe = 0, static_mode = 0, serializable = 1;
 
   if (!SAME_TYPE(SCHEME_TYPE(argv[0]), scheme_linklet_type))
     scheme_wrong_contract("recompile-linklet", "linklet?", 0, argc, argv);
@@ -463,10 +497,10 @@ static Scheme_Object *recompile_linklet(int argc, Scheme_Object **argv)
   }
 
   if (argc > 4)
-    parse_compile_options("recompile-linklet", 4, argc, argv, &unsafe, &static_mode);
+    parse_compile_options("recompile-linklet", 4, argc, argv, &unsafe, &static_mode, &serializable);
   
   linklet = compile_and_or_optimize_linklet(NULL, linklet, name, &import_keys, get_import,
-                                            unsafe, static_mode);
+                                            unsafe, static_mode, serializable);
 
   if (import_keys) {
     a[0] = (Scheme_Object *)linklet;
@@ -502,6 +536,18 @@ static Scheme_Object *eval_linklet(int argc, Scheme_Object **argv)
     }
   }
 
+#ifdef MZ_USE_JIT
+  if (linklet->native_lambdas) {
+    Scheme_Object *l;
+    l = linklet->native_lambdas;
+    linklet->native_lambdas = NULL;
+    while (SCHEME_PAIRP(l)) {
+      scheme_force_jit_generate((Scheme_Native_Lambda *)SCHEME_CAR(l));
+      l = SCHEME_CDR(l);
+    }
+  }
+#endif
+
   return (Scheme_Object *)linklet;
 }
 
@@ -569,7 +615,7 @@ static Scheme_Object *instantiate_linklet(int argc, Scheme_Object **argv)
     scheme_wrong_contract("instantiate-linklet", "linklet?", 0, argc, argv);
 
   l = argv[1];
-  while (!SCHEME_NULLP(l)) {
+  while (SCHEME_PAIRP(l)) {
     if (!SAME_TYPE(SCHEME_TYPE(SCHEME_CAR(l)), scheme_instance_type))
       break;
     l = SCHEME_CDR(l);
@@ -1112,7 +1158,7 @@ static Scheme_Hash_Tree *update_source_names(Scheme_Hash_Tree *source_names,
 static Scheme_Linklet *compile_and_or_optimize_linklet(Scheme_Object *form, Scheme_Linklet *linklet,
                                                        Scheme_Object *name,
                                                        Scheme_Object **_import_keys, Scheme_Object *get_import,
-                                                       int unsafe_mode, int static_mode)
+                                                       int unsafe_mode, int static_mode, int serializable)
 {
   Scheme_Config *config;
   int enforce_const, set_undef, can_inline;
@@ -1157,12 +1203,15 @@ static Scheme_Linklet *compile_and_or_optimize_linklet(Scheme_Object *form, Sche
 
   scheme_performance_record_end("compile", &perf_state);
 
+  if (serializable)
+    linklet->serializable = 1;
+
   return linklet;
 }
 
 Scheme_Linklet *scheme_compile_and_optimize_linklet(Scheme_Object *form, Scheme_Object *name)
 {
-  return compile_and_or_optimize_linklet(form, NULL, name, NULL, NULL, 0, 1);
+  return compile_and_or_optimize_linklet(form, NULL, name, NULL, NULL, 0, 1, 0);
 }
 
 /*========================================================================*/
@@ -1240,7 +1289,7 @@ Scheme_Object *scheme_linklet_run_finish(Scheme_Linklet* linklet, Scheme_Instanc
         /* Double-check that the definition-installing part of the
            continuation was not skipped. Otherwise, the compiler would
            not be able to assume that a variable reference that is
-           lexically later (incuding a reference to an imported
+           lexically later (including a reference to an imported
            variable) always references a defined variable. Putting the
            prompt around a definition's RHS might be a better
            approach, but that would change the language (so mabe next
