@@ -193,7 +193,8 @@ RKTIO_EXTERN rktio_fd_t *rktio_system_fd(rktio_t *rktio, intptr_t system_fd, int
    use `RKTIO_OPEN_INIT`. */
 
 RKTIO_EXTERN_NOERR intptr_t rktio_fd_system_fd(rktio_t *rktio, rktio_fd_t *rfd);
-/* Extracts a native file descriptor or socket. */
+/* Extracts a native file descriptor or socket. A file descriptor must
+   not be in pending-open mode as reported by `rktio_fd_is_pending_open`. */
 
 RKTIO_EXTERN rktio_bool_t rktio_fd_is_regular_file(rktio_t *rktio, rktio_fd_t *rfd);
 RKTIO_EXTERN rktio_bool_t rktio_fd_is_directory(rktio_t *rktio, rktio_fd_t *rfd);
@@ -203,8 +204,14 @@ RKTIO_EXTERN rktio_bool_t rktio_fd_is_terminal(rktio_t *rktio, rktio_fd_t *rfd);
 /* The functions mostly report values of recorded mode flags. */
 
 RKTIO_EXTERN rktio_bool_t rktio_fd_is_text_converted(rktio_t *rktio, rktio_fd_t *rfd);
-/* Reports whether RKTIO_OPEN_TEXT was use and has an effect. The
-   RKTIO_OPEN_TEXT flag has an effect only on Windows. */
+/* Reports whether `RKTIO_OPEN_TEXT` was use and has an effect. The
+   `RKTIO_OPEN_TEXT` flag has an effect only on Windows. */
+
+RKTIO_EXTERN rktio_bool_t rktio_fd_is_pending_open(rktio_t *rktio, rktio_fd_t *rfd);
+/* Reports whether `rfd` will block on writing because it corresponds
+   to the write end of a fifo that has no open reader. In that case,
+   `rktio_fd_system_fd` cannot report a file descriptor and `rktio_ltps_add`
+   will error with `RKTIO_ERROR_UNSUPPORTED`. */
 
 RKTIO_EXTERN_NOERR int rktio_fd_modes(rktio_t *rktio, rktio_fd_t *rfd);
 /* Returns all of the recorded mode flags, including those provided to
@@ -216,7 +223,9 @@ RKTIO_EXTERN rktio_fd_t *rktio_open(rktio_t *rktio, rktio_const_string_t src, in
    in read mode, and can report `RKTIO_ERROR_IS_A_DIRECTORY`,
    `RKTIO_ERROR_EXISTS`, or `RKTIO_ERROR_ACCESS_DENIED` in place of a
    system error in write mode. On Windows, can report
-   `RKTIO_ERROR_UNSUPPORTED_TEXT_MODE`. */
+   `RKTIO_ERROR_UNSUPPORTED_TEXT_MODE`. If `modes` has `RKTIO_OPEN_WRITE`
+   without `RKTIO_OPEN_READ`, then the result may be a file descriptor
+   in pending-open mode until the read end is opened. */
 
 RKTIO_EXTERN rktio_ok_t rktio_close(rktio_t *rktio, rktio_fd_t *fd);
 /* Can report `RKTIO_ERROR_EXISTS` in place of system error,
@@ -269,9 +278,9 @@ intptr_t rktio_write(rktio_t *rktio, rktio_fd_t *fd, const char *buffer, intptr_
    mode. Alternatively, the result can be `RKTIO_WRITE_ERROR` for an
    error. Although `rktio_write` is intended to write only bytes that
    can be fully delivered to the OS, there may be OS limitations that
-   require buffering (e.g., on Windows). Use `rktio_poll_write_flushed`
-   to make sure the data is received by the destination before closing
-   `fd`. */
+   require buffering (e.g., on ancient versions of Windows). Use
+   `rktio_poll_write_flushed` to make sure the data is received by the
+   destination before closing `fd`. */
 
 #define RKTIO_WRITE_ERROR (-2)
 
@@ -466,7 +475,7 @@ RKTIO_EXTERN_ERR(RKTIO_WRITE_ERROR)
 intptr_t rktio_udp_sendto(rktio_t *rktio, rktio_fd_t *rfd, RKTIO_NULLABLE rktio_addrinfo_t *addr,
                           const char *buffer, intptr_t len);
 /* Extends `rktio_write` to accept a destination `addr`, and binds `rfd` if it 
-   is not bound aready. The `addr` can be NULL if the socket is connected. */
+   is not bound already. The `addr` can be NULL if the socket is connected. */
 
 RKTIO_EXTERN_ERR(RKTIO_WRITE_ERROR)
 intptr_t rktio_udp_sendto_in(rktio_t *rktio, rktio_fd_t *rfd, RKTIO_NULLABLE rktio_addrinfo_t *addr,
@@ -490,9 +499,14 @@ RKTIO_EXTERN rktio_length_and_addrinfo_t *rktio_udp_recvfrom_in(rktio_t *rktio,
 
 RKTIO_EXTERN rktio_ok_t rktio_udp_set_receive_buffer_size(rktio_t *rktio, rktio_fd_t *rfd, int size);
 
+RKTIO_EXTERN rktio_ok_t rktio_udp_set_ttl(rktio_t *rktio, rktio_fd_t *rfd, int ttl_val);
+RKTIO_EXTERN_ERR(RKTIO_PROP_ERROR) rktio_tri_t rktio_udp_get_ttl(rktio_t *rktio, rktio_fd_t *rfd);
+
+
 RKTIO_EXTERN_ERR(RKTIO_PROP_ERROR) rktio_tri_t rktio_udp_get_multicast_loopback(rktio_t *rktio, rktio_fd_t *rfd);
 RKTIO_EXTERN rktio_ok_t rktio_udp_set_multicast_loopback(rktio_t *rktio, rktio_fd_t *rfd, rktio_bool_t on);
 RKTIO_EXTERN_ERR(RKTIO_PROP_ERROR) rktio_tri_t rktio_udp_get_multicast_ttl(rktio_t *rktio, rktio_fd_t *rfd);
+
 RKTIO_EXTERN rktio_ok_t rktio_udp_set_multicast_ttl(rktio_t *rktio, rktio_fd_t *rfd, int ttl_val);
 
 #define RKTIO_PROP_ERROR (-2)
@@ -527,7 +541,7 @@ RKTIO_EXTERN rktio_bool_t rktio_is_ok_envvar_name(rktio_t *rktio, rktio_const_st
 RKTIO_EXTERN rktio_bool_t rktio_are_envvar_names_case_insensitive(rktio_t *rktio);
 /* Checks whether environment variables are case-folded by the OS.
    That doesn't mean that clients need to case-fold names, but clients
-   may want to immitate the OS. */
+   may want to imitate the OS. */
 
 RKTIO_EXTERN char *rktio_getenv(rktio_t *rktio, rktio_const_string_t name);
 /* Gets an environment variable value, or reports
@@ -582,7 +596,8 @@ RKTIO_EXTERN rktio_process_result_t *rktio_process(rktio_t *rktio,
                                                    rktio_const_string_t current_directory,
                                                    rktio_envvars_t *envvars,
                                                    int flags);
-/* `flags` flags: */
+/* The output file descriptors `stdin_fd` must not be a pending-open
+   descriptor. The `flags` are: */
 #define RKTIO_PROCESS_NEW_GROUP                 (1<<0)
 #define RKTIO_PROCESS_STDOUT_AS_STDERR          (1<<1)
 #define RKTIO_PROCESS_WINDOWS_EXACT_CMDLINE     (1<<2)
@@ -695,7 +710,7 @@ RKTIO_EXTERN void rktio_poll_set_add_eventmask(rktio_t *rktio, rktio_poll_set_t
 RKTIO_EXTERN void rkio_reset_sleep_backoff(rktio_t *rktio);
 /* Call this function when using `rktio_poll_set_add_eventmask` and
    when matching events are not always consumed from the queue between
-   sleeps. To accomodate messages that are not consumed, the poll set
+   sleeps. To accommodate messages that are not consumed, the poll set
    will actually only sleep a short while at first, and then back off
    exponentially. Call this function when your program does useful
    work (instead of spinning on sleep) to reset the backoff
@@ -998,14 +1013,21 @@ typedef struct rktio_date_t {
   char *zone_name; /* can be NULL; otherwise, free it */
 } rktio_date_t;
 
-RKTIO_EXTERN_NOERR intptr_t rktio_get_milliseconds(void);
-RKTIO_EXTERN_NOERR double rktio_get_inexact_milliseconds(void);
+RKTIO_EXTERN_NOERR uintptr_t rktio_get_milliseconds(void);
+/* Overflow may cause the result to wrap around to 0, at least on a
+   32-bit platform. */
 
-RKTIO_EXTERN_NOERR intptr_t rktio_get_process_milliseconds(rktio_t *rktio);
-RKTIO_EXTERN_NOERR intptr_t rktio_get_process_children_milliseconds(rktio_t *rktio);
+RKTIO_EXTERN_NOERR double rktio_get_inexact_milliseconds(void);
+/* No overflow, but won't strictly increase if the system clock is reset. */
+
+RKTIO_EXTERN_NOERR uintptr_t rktio_get_process_milliseconds(rktio_t *rktio);
+RKTIO_EXTERN_NOERR uintptr_t rktio_get_process_children_milliseconds(rktio_t *rktio);
+/* Overflow may cause the result to wrap around to 0, at least on a
+   32-bit platform. */
 
 RKTIO_EXTERN_NOERR rktio_timestamp_t rktio_get_seconds(rktio_t *rktio);
 RKTIO_EXTERN rktio_date_t *rktio_seconds_to_date(rktio_t *rktio, rktio_timestamp_t seconds, int nanoseconds, int get_gmt);
+/* A timestamp can be negative to represent a date before 1970. */
 
 /*************************************************/
 /* Windows ShellExecute                          */
@@ -1119,6 +1141,9 @@ RKTIO_EXTERN rktio_convert_result_t *rktio_convert_in(rktio_t *rktio,
    doesn't return a NULL result; instead, `converted` in the result
    reports the error. */
 
+RKTIO_EXTERN void rktio_convert_reset(rktio_t *rktio, rktio_converter_t *cvt);
+/* Resets a converter to its initial state. */
+
 RKTIO_EXTERN_NOERR char *rktio_locale_recase(rktio_t *rktio,
                                              rktio_bool_t to_up,
                                              rktio_const_string_t in);
@@ -1162,7 +1187,7 @@ RKTIO_EXTERN void rktio_set_locale(rktio_t *rktio, rktio_const_string_t name);
 
 RKTIO_EXTERN_NOERR char *rktio_push_c_numeric_locale(rktio_t *rktio);
 RKTIO_EXTERN void rktio_pop_c_numeric_locale(rktio_t *rktio, char *prev);
-/* Use this pair of funtions to temporarily switch the locale to the C
+/* Use this pair of functions to temporarily switch the locale to the C
    locale for number parsing and printing. The result of the first
    function is deallocated when passed to second function. */
 
@@ -1228,15 +1253,27 @@ RKTIO_EXTERN rktio_dll_t *rktio_dll_open(rktio_t *rktio, rktio_const_string_t na
    reported, then rktio_dll_get_error() must be used before any other
    `rktio_dll_...` call to get an error string.
 
-   Currently, there's no way to close and unload a DLL. Even when the
-   given `rktio` is closed with `rktio_destroy`, loaded libraries
-   remain in the process. */
+   If a DLL has been loaded with `name` already, the previous result
+   is returned again, but with an internal reference count returned.
+   The `as_global` argument matters only for the first load of a DLL
+   thrrough a given `name`.
+
+   Unless the DLL is explicitly unloaded with `rktio_dll_close`, even
+   when the given `rktio` is closed with `rktio_destroy`, loaded
+   libraries remain in the process. */
 
 RKTIO_EXTERN void *rktio_dll_find_object(rktio_t *rktio, rktio_dll_t *dll, rktio_const_string_t name);
 /* Find an address within `dll` for the `name` export.
 
    An error result can be `RKTIO_ERROR_DLL` as for `rktio_dll_open`. */
 
+RKTIO_EXTERN rktio_ok_t rktio_dll_close(rktio_t *rktio, rktio_dll_t *dll);
+/* Decrements the reference count on `dll`, and if it goes to zero,
+   unloads the DLL using system-provided functions and destroys the
+   `dll` argument.
+
+   An error result can be `RKTIO_ERROR_DLL` as for `rktio_dll_open`. */
+
 RKTIO_EXTERN char *rktio_dll_get_error(rktio_t *rktio);
 /* Returns an error for a previous `rktio_dll_...` call, or NULL
    if no error string is available or has already been returned.
@@ -1244,7 +1281,10 @@ RKTIO_EXTERN char *rktio_dll_get_error(rktio_t *rktio);
 
 typedef void *(*dll_open_proc)(rktio_const_string_t name, rktio_bool_t as_global);
 typedef void *(*dll_find_object_proc)(void *h, rktio_const_string_t name);
-RKTIO_EXTERN void rktio_set_dll_procs(dll_open_proc dll_open, dll_find_object_proc dll_find_object);
+typedef void (*dll_close_proc)(void *h);
+RKTIO_EXTERN void rktio_set_dll_procs(dll_open_proc dll_open,
+                                      dll_find_object_proc dll_find_object,
+                                      dll_close_proc dll_close);
 /* Installs procedures that are tried before native mechanisms,
    currently only supported for Windows. */
 
