@@ -29,6 +29,9 @@ struct rktio_fd_t {
   int bufcount;
   char buffer[1];
 # endif
+# ifdef RKTIO_USE_PENDING_OPEN
+  struct open_in_thread_t *pending;
+# endif
 #endif
   
 #ifdef RKTIO_SYSTEM_WINDOWS
@@ -105,6 +108,7 @@ static void deinit_fd(rktio_fd_t *rfd, int full_close)
 
 #define DELAYED_CONSOLE_HANDLE ((HANDLE)-2)
 static void force_console(rktio_fd_t *rfd);
+static HANDLE get_std_handle(int which);
 
 static long WINAPI WindowsFDReader(Win_FD_Input_Thread *th);
 static void WindowsFDICleanup(Win_FD_Input_Thread *th, int close_mode);
@@ -201,9 +205,46 @@ rktio_fd_t *rktio_system_fd(rktio_t *rktio, intptr_t system_fd, int modes)
   return rfd;
 }
 
+int rktio_fd_is_pending_open(rktio_t *rktio, rktio_fd_t *rfd)
+{
+#ifdef RKTIO_USE_PENDING_OPEN
+  if (rfd->pending)
+    rktio_pending_open_poll(rktio, rfd, rfd->pending);
+  return !!rfd->pending;
+#else
+  return 0;
+#endif
+}
+
+#ifdef RKTIO_USE_PENDING_OPEN
+rktio_fd_t *rktio_pending_system_fd(rktio_t *rktio, struct open_in_thread_t *oit, int modes)
+{
+  rktio_fd_t *rfd;
+
+  rfd = calloc(1, sizeof(rktio_fd_t));
+  rfd->modes = (modes - (modes & RKTIO_OPEN_INIT));
+  rfd->pending = oit;
+
+  return rfd;
+}
+
+void rktio_update_system_fd(rktio_t *rktio, rktio_fd_t *rfd, int fd, int modes)
+/* Convert an open-pending rfd to a normal rfd. The given `modes` adds
+   to (and completes) the modes provided to `rktio_pending_system_fd`. */
+{
+  rfd->fd = fd;
+  rfd->modes |= modes;
+  rfd->pending = NULL;
+}
+#endif
+
+
 intptr_t rktio_internal_fd_system_fd(rktio_fd_t *rfd)
 {
 #ifdef RKTIO_SYSTEM_UNIX
+# ifdef RKTIO_USE_PENDING_OPEN
+  if (rfd->pending) return -1;
+# endif
   return rfd->fd;
 #endif
 #ifdef RKTIO_SYSTEM_WINDOWS
@@ -242,7 +283,7 @@ rktio_fd_t *rktio_std_fd(rktio_t *rktio, int which)
     which = STD_ERROR_HANDLE;
     break;
   }
-  h = GetStdHandle(which);
+  h = get_std_handle(which);
   if ((h == INVALID_HANDLE_VALUE) || (h == NULL)) {
     h = DELAYED_CONSOLE_HANDLE; /* => open a console on demand */
   }
@@ -272,6 +313,14 @@ int rktio_fd_is_udp(rktio_t *rktio, rktio_fd_t *rfd)
   return ((rfd->modes & RKTIO_OPEN_UDP) ? 1 : 0);
 }
 
+int rktio_fd_is_open_pending(rktio_t *rktio, rktio_fd_t *rfd)
+{
+#ifdef RKTIO_USE_PENDING_OPEN
+  if (rfd->pending) return 1;
+#endif
+  return 0;
+}
+
 int rktio_fd_modes(rktio_t *rktio, rktio_fd_t *rfd)
 {
   return rfd->modes;
@@ -298,6 +347,9 @@ int rktio_system_fd_is_terminal(rktio_t *rktio, intptr_t fd)
 
 int rktio_fd_is_terminal(rktio_t *rktio, rktio_fd_t *rfd)
 {
+#ifdef RKTIO_USE_PENDING_OPEN
+  if (rfd->pending) return 0;
+#endif
   return rktio_system_fd_is_terminal(rktio, (intptr_t)rfd->fd);
 }
 
@@ -315,6 +367,13 @@ rktio_fd_t *rktio_dup(rktio_t *rktio, rktio_fd_t *rfd)
 #ifdef RKTIO_SYSTEM_UNIX
   intptr_t nfd;
 
+# ifdef RKTIO_USE_PENDING_OPEN
+  if (rfd->pending) {
+    rktio_pending_open_retain(rktio, rfd->pending);
+    return rktio_pending_system_fd(rktio, rfd->pending, rfd->modes);
+  }
+# endif
+
   do {
     nfd = dup(rfd->fd);
   } while (nfd == -1 && errno == EINTR);
@@ -375,16 +434,22 @@ void rktio_reliably_close(intptr_t s)
 #endif
 
 static rktio_ok_t do_close(rktio_t *rktio /* maybe NULL */, rktio_fd_t *rfd, int set_error)
+/* The `rktio` argument can be NULL for a detached file descriptor */
 {
   int ok;
 
 #ifdef RKTIO_SYSTEM_UNIX
   int cr;
 
-  cr = rktio_reliably_close_err(rfd->fd);
+# ifdef RKTIO_USE_PENDING_OPEN
+  if (rfd->pending)
+    cr = rktio_pending_open_release(rktio, rfd->pending);
+  else
+# endif
+    cr = rktio_reliably_close_err(rfd->fd);
 
 # ifdef RKTIO_USE_FCNTL_AND_FORK_FOR_FILE_LOCKS
-  if (rktio && !(rfd->modes & RKTIO_OPEN_SOCKET))
+  if (rktio && !(rfd->modes & RKTIO_OPEN_SOCKET) && (!cr || !set_error))
     rktio_release_lockf(rktio, rfd->fd);
 # endif
 
@@ -446,12 +511,24 @@ void rktio_forget(rktio_t *rktio, rktio_fd_t *rfd)
 
 rktio_fd_transfer_t *rktio_fd_detach(rktio_t *rktio, rktio_fd_t *rfd)
 {
+#ifdef RKTIO_USE_PENDING_OPEN
+  if (rfd->pending)
+    rktio_pending_open_detach(rktio, rfd->pending);
+#endif
+
   return (rktio_fd_transfer_t *)rfd;
 }
 
 rktio_fd_t *rktio_fd_attach(rktio_t *rktio, rktio_fd_transfer_t *rfdt)
 {
-  return (rktio_fd_t *)rfdt;
+  rktio_fd_t *rfd = (rktio_fd_t *)rfdt;
+
+#ifdef RKTIO_USE_PENDING_OPEN
+  if (rfd->pending)
+    rktio_pending_open_attach(rktio, rfd->pending);
+#endif
+
+  return rfd;
 }
 
 void rktio_fd_close_transfer(rktio_fd_transfer_t *rfdt)
@@ -592,6 +669,19 @@ int poll_write_ready_or_flushed(rktio_t *rktio, rktio_fd_t *rfd, int check_flush
     return RKTIO_POLL_READY;
   else {
     int sr;
+
+# ifdef RKTIO_USE_PENDING_OPEN
+    if (rfd->pending) {
+      int errval = rktio_pending_open_poll(rktio, rfd, rfd->pending);
+      if (errval) {
+        errno = errval;
+        get_posix_error();
+        return RKTIO_POLL_ERROR;
+      } else if (rfd->pending)
+        return RKTIO_POLL_NOT_READY;
+    }
+# endif
+
 # ifdef HAVE_POLL_SYSCALL
     struct pollfd pfd[1];
     pfd[0].fd = rfd->fd;
@@ -616,7 +706,7 @@ int poll_write_ready_or_flushed(rktio_t *rktio, rktio_fd_t *rfd, int check_flush
       /* Mac OS X 10.8 and 10.9: select() seems to claim that a pipe
          is always ready for output. To work around that problem,
          kqueue() support might be used for pipes, but that has different
-         problems. The poll() code above should be used, instead. */
+         problems. The pol() code above should be used, instead. */
       sr = select(rfd->fd + 1, NULL, RKTIO_FDS(writefds), RKTIO_FDS(exnfds), &time);
     } while ((sr == -1) && (errno == EINTR));
 # endif
@@ -643,6 +733,12 @@ int poll_write_ready_or_flushed(rktio_t *rktio, rktio_fd_t *rfd, int check_flush
     int retval;
     Win_FD_Output_Thread *oth = rfd->oth;
 
+    if (check_flushed && rfd->oth->nonblocking) {
+      /* Not Windows 95, so any written data really is in the pipe, as
+         good as flushed, and we don't really need to ask the thread. */
+      return RKTIO_POLL_READY;
+    }
+
     WaitForSingleObject(oth->lock_sema, INFINITE);
     if (oth->nonblocking) {
       if (oth->needflush) {
@@ -693,6 +789,13 @@ void rktio_poll_add(rktio_t *rktio, rktio_fd_t *rfd, rktio_poll_set_t *fds, int
 #ifdef RKTIO_SYSTEM_UNIX
   rktio_poll_set_t *fds2;
 
+# ifdef RKTIO_USE_PENDING_OPEN
+  if (rfd->pending) {
+    rktio_poll_add_pending_open(rktio, rfd, rfd->pending, fds);
+    return;
+  }
+# endif
+
   if (modes & RKTIO_POLL_READ) {
     RKTIO_FD_SET(rfd->fd, fds);
   }
@@ -804,15 +907,21 @@ intptr_t rktio_read_converted(rktio_t *rktio, rktio_fd_t *rfd, char *buffer, int
     else
       return bc;
   } else {
-    /* We use a non-blocking read here, even though we've waited
-       for input above, because an external process might have
-       gobbled the characters that we expected to get. */
     int old_flags;
+
+    /* Check that input is available. This step is needed
+       for fifos to avoid getting an EOF-looking result
+       if there's no writer, yet. */
+    if (!rktio_poll_read_ready(rktio, rfd))
+      return 0;
     
     old_flags = fcntl(rfd->fd, F_GETFL, 0);
     if (!(old_flags & RKTIO_NONBLOCKING))
       fcntl(rfd->fd, F_SETFL, old_flags | RKTIO_NONBLOCKING);
     
+    /* We use a non-blocking read here, even though we've waited
+       for input above, because an external process might have
+       gobbled the characters that we expected to get. */
     do {
       bc = read(rfd->fd, buffer, len);
     } while ((bc == -1) && errno == EINTR);
@@ -1226,7 +1335,19 @@ intptr_t rktio_write(rktio_t *rktio, rktio_fd_t *rfd, const char *buffer, intptr
 
   if (rfd->modes & RKTIO_OPEN_SOCKET)
     return rktio_socket_write(rktio, rfd, buffer, len);
-  
+
+# ifdef RKTIO_USE_PENDING_OPEN
+  if (rfd->pending) {
+    int errval = rktio_pending_open_poll(rktio, rfd, rfd->pending);
+    if (errval) {
+      errno = errval;
+      get_posix_error();
+      return RKTIO_WRITE_ERROR;
+    } else if (rfd->pending)
+      return 0;
+  }
+# endif
+
   flags = fcntl(rfd->fd, F_GETFL, 0);
   if (!(flags & RKTIO_NONBLOCKING))
     fcntl(rfd->fd, F_SETFL, flags | RKTIO_NONBLOCKING);
@@ -1303,7 +1424,7 @@ intptr_t rktio_write(rktio_t *rktio, rktio_fd_t *rfd, const char *buffer, intptr
 	if (towrite)
 	  ok = WriteConsoleW((HANDLE)rfd->fd, w_buffer, towrite, &winwrote, NULL);
 	else {
-	  /* can happend if can_leftover is > 0 */
+	  /* can happen if can_leftover is > 0 */
 	  ok = 1;
 	  winwrote = 0;
 	}
@@ -1865,6 +1986,31 @@ static void WindowsFDOCleanup(Win_FD_Output_Thread *oth, int close_mode)
 
 #ifdef RKTIO_SYSTEM_WINDOWS
 
+#ifndef ENABLE_VIRTUAL_TERMINAL_PROCESSING
+# define ENABLE_VIRTUAL_TERMINAL_PROCESSING  0x4
+#endif
+
+static HANDLE get_std_handle(int which) {
+  HANDLE h;
+
+  h = GetStdHandle(which);
+
+  if ((h == INVALID_HANDLE_VALUE) || (h == NULL))
+    return h;
+
+  if ((which == STD_OUTPUT_HANDLE) || (which == STD_ERROR_HANDLE)) {
+    if (GetFileType(h) == FILE_TYPE_CHAR) {
+      /* Try to enable ANSI escape codes, which should work for a recent
+         enough version of Windows */
+      DWORD mode = 0;
+      GetConsoleMode(h, &mode);
+      SetConsoleMode(h, mode | ENABLE_VIRTUAL_TERMINAL_PROCESSING);
+    }
+  }
+
+  return h;
+}
+
 static void force_console(rktio_fd_t *rfd) {
   /* DELAYED_CONSOLE_HANDLE is used to indicate that a console should be created on demand */
   if (rfd->fd == DELAYED_CONSOLE_HANDLE) {
@@ -1877,7 +2023,7 @@ static void force_console(rktio_fd_t *rfd) {
       which = STD_INPUT_HANDLE;
     else
       which = STD_OUTPUT_HANDLE;
-    h = GetStdHandle(which);
+    h = get_std_handle(which);
 
     rfd->fd = h;
   }
