@@ -629,8 +629,10 @@ char *rktio_get_current_directory(rktio_t *rktio)
    } else
      break;
  }
- if (!r)
+ if (!r) {
+   free(s);
    get_posix_error();
+ }
  return r;
 #endif
 }
@@ -887,6 +889,7 @@ char *rktio_readlink(rktio_t *rktio, const char *fullfilename)
           set_racket_error(RKTIO_ERROR_NOT_A_LINK);
         else
           get_posix_error();
+        free(buffer);
         return NULL;
       }
     } else if (len == buf_len) {
@@ -915,7 +918,8 @@ int rktio_make_directory(rktio_t *rktio, const char *filename)
   /* Make sure path doesn't have trailing separator: */
   len = strlen(filename);
   while (len && IS_A_SEP(filename[len - 1])) {
-    copied = MSC_IZE(strdup)(filename);
+    if (!copied)
+      copied = MSC_IZE(strdup)(filename);
     copied[--len] = 0;
     filename = copied;
   }
@@ -1483,12 +1487,13 @@ char *rktio_directory_list_step(rktio_t *rktio, rktio_directory_list_t *dl)
   while ((e = readdir(dl->dir))) {
     int nlen;
 
-# ifdef DIRENT_NO_NAMLEN
-    nlen = strlen(e->d_name);
-# elif defined(__QNX__) || defined(__QNXNTO__)
+# ifdef HAVE_DIRENT_NAMLEN
+    nlen = e->d_namlen;
+# elif HAVE_DIRENT_NAMELEN
+    /* Case for QNX - which seems to define d_namelen instead */
     nlen = e->d_namelen;
 # else
-    nlen = e->d_namlen;
+    nlen = strlen(e->d_name);
 # endif
 
 # if defined(RKTIO_SYSTEM_UNIX) || defined(RKTIO_SYSTEM_WINDOWS)
@@ -1944,8 +1949,10 @@ char *rktio_system_path(rktio_t *rktio, int which)
 #else      
       return append_paths(home, "racket-prefs.rktd", 1, 0);
 #endif
-    } else
+    } else {
+      free(home);
       return strdup("/");
+    }
   }
 #endif
 
